# @Author: BingWu Yang <detailyang>
# @Date:   2016-03-30T17:00:32+08:00
# @Email:  detailyang@gmail.com
# @Last modified by:   detailyang
# @Last modified time: 2016-03-31T13:08:36+08:00
# @License: The MIT License (MIT)


import re


keypattern = re.compile('--[qhb](.*)')


class ESLNode:
    """ Abstract base class for AST nodes.
    """
    def children(self):
        pass


class RequestNode(ESLNode):
    def __init__(self, method, left, right):
        self.method =method
        self.left = left
        self.right = right

    def children(self):
        nodelist = []
        if self.left is not None:
                nodelist.append(("left", self.left))
        if self.right is not None:
                nodelist.append(("right", self.right))
        return tuple(nodelist)


class MethodNode(ESLNode):
    def __init__(self, method):
        self.name = method.upper() or 'GET'


class URLNode(ESLNode):
    def __init__(self, url):
        self.url = url


class HeaderNode(ESLNode):
    def __init__(self, header):
        r = keypattern.match(header)
        self.key = r.group(1)

class QueryStringNode(ESLNode):
    def __init__(self, qs):
        r = keypattern.match(qs)
        self.key = r.group(1)


class BodyNode(ESLNode):
    def __init__(self, body):
        r = keypattern.match(body)
        self.key = r.group(1)


class ValueNode(ESLNode):
    def __init__(self, value):
        self.value = value[1:] if value.startswith('=') else value


class ShellNode(ESLNode):
    def __init__(self, value):
        self.value = value[3:-1] if value.startswith('=!(') and value.endswith(')') else value


class OptionListNode(ESLNode):
    def __init__(self, options):
        self.options = options or []

    def children(self):
        nodelist = []
        for i, child in enumerate(self.options or []):
            nodelist.append(('options[%d]' % i, child))
        return tuple(nodelist)

    def append(self, option):
        self.options.append(option)
        return self


class OptionNode(ESLNode):
    def __init__(self, key, value):
        self.key = key
        self.value = value



# @Author: BingWu Yang <detailyang>
# @Date:   2016-03-29T17:47:44+08:00
# @Email:  detailyang@gmail.com
# @Last modified by:   detailyang
# @Last modified time: 2016-03-31T13:12:43+08:00
# @License: The MIT License (MIT)


import ply.yacc as yacc
import eslast as ast

from esllexer import ESLLexer


tokens = ESLLexer.tokens


def p_request(p):
    '''request : URL
           | URL METHOD
           | URL METHOD OPTIONS'''
    if len(p) == 2:
        p[0] = ast.RequestNode(ast.MethodNode('GET'), ast.URLNode(p[1]), None)
    elif len(p) == 3:
        p[0] = ast.RequestNode(ast.MethodNode(p[2]), ast.URLNode(p[1]), None)
    else:
        p[0] = ast.RequestNode(ast.MethodNode(p[2]), ast.URLNode(p[1]), p[3])

def p_options(p):
    '''OPTIONS :
              | OPTION
              | OPTIONS OPTION'''
    if len(p)  == 2:
        p[0] = ast.OptionListNode([p[1]])
    else:
        p[0] = p[1].append(p[2])

def p_option_empty(p):
    ' OPTION :   empty '
    p[0] = p[1]

def p_option_header(p):
    ' OPTION :   HEADERVALUE '
    p[0] = p[1]

def p_option_querystring(p):
    ' OPTION :  QUERYSTRINGVALUE '
    p[0] = p[1]

def p_option_body(p):
    ' OPTION : BODYVALUE '
    p[0] = p[1]

def p_empty(p):
    'empty :'
    p[0] = []

def p_querystring_value(p):
    '''QUERYSTRINGVALUE : QUERYSTRING VALUE '''
    p[0] = ast.OptionNode(ast.QueryStringNode(p[1]), ast.ValueNode(p[2]))

def p_querystring_shell(p):
    '''QUERYSTRINGVALUE : QUERYSTRING SHELL '''
    p[0] = ast.OptionNode(ast.QueryStringNode(p[1]), ast.ShellNode(p[2]))

def p_header_value(p):
    '''HEADERVALUE : HEADER VALUE '''
    p[0] = ast.OptionNode(ast.HeaderNode(p[1]), ast.ValueNode(p[2]))

def p_header_shell(p):
    '''HEADERVALUE : HEADER SHELL '''
    p[0] = ast.OptionNode(ast.HeaderNode(p[1]), ast.ShellNode(p[2]))

def p_body_value(p):
    '''BODYVALUE : BODY VALUE '''
    p[0] = ast.OptionNode(ast.HeaderNode(p[1]), ast.ValueNode(p[2]))

def p_body_shell(p):
    '''BODYVALUE : BODY SHELL '''
    p[0] = ast.OptionNode(ast.HeaderNode(p[1]), ast.ShellNode(p[2]))

def p_error(p):
    print(p)
    print("Syntax Error")
    print("ESL format: {URL} {METHOD} {OPTIONS}")
    print("{URL}: https://example.com|examples.com|/api/endpoints")
    print("{METHOD}: GET|get|POST|post|DELETE|delete|PUT|put")
    print("{OPTIONS}: --hContent-Type=application/json")
    print("{OPTIONS}: --qper_page=1")
    print("{OPTIONS}: --busername=xxxx")

def parse(text):
    parser = yacc.yacc(debug=True)
    ast = parser.parse(text, ESLLexer().build())
    return ast

if __name__ == '__main__':
    ast = parse("/api/cmdb/peoples/ get --qhost_ip=!(ifconfig eth0) --qhost_name=bj-sdf --hContent-Type=abcd --bslkjsdf=123")     # Test it
    print(ast.left)
    print(ast.method)
    for option in ast.right.options:
        key = option.key
        value = option.value




# parsetab.py
# This file is automatically generated. Do not edit.
_tabversion = '3.8'

_lr_method = 'LALR'

_lr_signature = '17A76441A771B4A3D86C362AEBE88676'
    
_lr_action_items = {'BODY':([3,5,7,8,10,11,12,13,14,15,16,17,18,19,],[4,-8,-9,4,-10,-7,-5,-17,-16,-13,-12,-6,-15,-14,]),'SHELL':([4,6,9,],[13,15,18,]),'URL':([0,],[1,]),'QUERYSTRING':([3,5,7,8,10,11,12,13,14,15,16,17,18,19,],[6,-8,-9,6,-10,-7,-5,-17,-16,-13,-12,-6,-15,-14,]),'VALUE':([4,6,9,],[14,16,19,]),'HEADER':([3,5,7,8,10,11,12,13,14,15,16,17,18,19,],[9,-8,-9,9,-10,-7,-5,-17,-16,-13,-12,-6,-15,-14,]),'METHOD':([1,],[3,]),'$end':([1,2,3,5,7,8,10,11,12,13,14,15,16,17,18,19,],[-1,0,-2,-8,-9,-3,-10,-7,-5,-17,-16,-13,-12,-6,-15,-14,]),}

_lr_action = {}
for _k, _v in _lr_action_items.items():
   for _x,_y in zip(_v[0],_v[1]):
      if not _x in _lr_action:  _lr_action[_x] = {}
      _lr_action[_x][_k] = _y
del _lr_action_items

_lr_goto_items = {'OPTION':([3,8,],[12,17,]),'request':([0,],[2,]),'QUERYSTRINGVALUE':([3,8,],[7,7,]),'BODYVALUE':([3,8,],[10,10,]),'OPTIONS':([3,],[8,]),'empty':([3,8,],[11,11,]),'HEADERVALUE':([3,8,],[5,5,]),}

_lr_goto = {}
for _k, _v in _lr_goto_items.items():
   for _x, _y in zip(_v[0], _v[1]):
       if not _x in _lr_goto: _lr_goto[_x] = {}
       _lr_goto[_x][_k] = _y
del _lr_goto_items
_lr_productions = [
  ("S' -> request","S'",1,None,None,None),
  ('request -> URL','request',1,'p_request','eslyacc.py',27),
  ('request -> URL METHOD','request',2,'p_request','eslyacc.py',28),
  ('request -> URL METHOD OPTIONS','request',3,'p_request','eslyacc.py',29),
  ('OPTIONS -> <empty>','OPTIONS',0,'p_options','eslyacc.py',38),
  ('OPTIONS -> OPTION','OPTIONS',1,'p_options','eslyacc.py',39),
  ('OPTIONS -> OPTIONS OPTION','OPTIONS',2,'p_options','eslyacc.py',40),
  ('OPTION -> empty','OPTION',1,'p_option_empty','eslyacc.py',47),
  ('OPTION -> HEADERVALUE','OPTION',1,'p_option_header','eslyacc.py',51),
  ('OPTION -> QUERYSTRINGVALUE','OPTION',1,'p_option_querystring','eslyacc.py',55),
  ('OPTION -> BODYVALUE','OPTION',1,'p_option_body','eslyacc.py',59),
  ('empty -> <empty>','empty',0,'p_empty','eslyacc.py',63),
  ('QUERYSTRINGVALUE -> QUERYSTRING VALUE','QUERYSTRINGVALUE',2,'p_querystring_value','eslyacc.py',67),
  ('QUERYSTRINGVALUE -> QUERYSTRING SHELL','QUERYSTRINGVALUE',2,'p_querystring_shell','eslyacc.py',71),
  ('HEADERVALUE -> HEADER VALUE','HEADERVALUE',2,'p_header_value','eslyacc.py',75),
  ('HEADERVALUE -> HEADER SHELL','HEADERVALUE',2,'p_header_shell','eslyacc.py',79),
  ('BODYVALUE -> BODY VALUE','BODYVALUE',2,'p_body_value','eslyacc.py',83),
  ('BODYVALUE -> BODY SHELL','BODYVALUE',2,'p_body_shell','eslyacc.py',87),
]



# @Author: BingWu Yang <detailyang>
# @Date:   2016-03-30T20:27:36+08:00
# @Email:  detailyang@gmail.com
# @Last modified by:   detailyang
# @Last modified time: 2016-03-31T18:41:22+08:00
# @License: The MIT License (MIT)


from setuptools import setup, find_packages

setup(
    name = 'esl',
    version = '0.1.0',
    keywords = ('esl', 'eva-shell-language'),
    description = 'eva shell language for eva-shell',
    license = 'MIT License',
    install_requires = ['ply>=3.8'],

    author = 'detailyang',
    author_email = 'detailyang@gmail.com',

    packages = find_packages(),
    platforms = 'any',
)
